"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolvePathRelativeToConfig = resolvePathRelativeToConfig;
exports.resolveProjectName = resolveProjectName;
exports.resolveSchemaPath = resolveSchemaPath;
exports.getGraphQLProjects = getGraphQLProjects;
exports.getGraphQLSchemaPaths = getGraphQLSchemaPaths;
exports.getGraphQLProjectIncludedFilePaths = getGraphQLProjectIncludedFilePaths;
exports.getGraphQLProjectForSchemaPath = getGraphQLProjectForSchemaPath;
exports.defaultGraphQLProjectName = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _fs = require("fs");

var _util = require("util");

var _glob = _interopRequireDefault(require("glob"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var defaultGraphQLProjectName = 'GraphQL'; // temporary utility until `graphql-config` supports this function natively
// see: https://github.com/prisma/graphql-config/pull/113

exports.defaultGraphQLProjectName = defaultGraphQLProjectName;

function resolvePathRelativeToConfig(project, relativePath) {
  return project.resolveConfigPath(relativePath);
}

function resolveProjectName(project) {
  var defaultName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultGraphQLProjectName;
  return project.projectName || defaultName;
}

function resolveSchemaPath(project) {
  var ignoreMissing = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  // schemaPath is nullable in graphq-config even though it cannot actually be
  // omitted. This function simplifies access to the schemaPath without
  // requiring a type guard.
  if (!project.schemaPath) {
    // this case should never happen with a properly formatted config file.
    // graphql-config currently does not perform any validation so it's possible
    // for a mal-formed schema to be loaded at runtime.
    throw new Error("Missing GraphQL schemaPath for project '".concat(resolveProjectName(project), "'"));
  } // resolve fully qualified schemaPath


  var schemaPath = project.resolveConfigPath(project.schemaPath);

  if (ignoreMissing) {
    return schemaPath;
  }

  if (!(0, _fs.existsSync)(schemaPath)) {
    var forProject = project.projectName ? " for project '".concat(project.projectName, "'") : '';
    throw new Error(["Schema not found".concat(forProject, "."), "Expected to find the schema at '".concat(schemaPath, "' but the path does not exist."), "Check '".concat(project.configPath, "' and verify that schemaPath is configured correctly").concat(forProject, ".")].join(' '));
  }

  return schemaPath;
}

function getGraphQLProjects(config) {
  var projects = config.getProjects();

  if (projects) {
    // multi-project configuration, return an array of projects
    return Object.values(projects);
  }

  var project = config.getProjectConfig();

  if (project && project.schemaPath) {
    // single project configuration, return an array of the single project
    return [project];
  } // invalid project configuration


  throw new Error("No projects defined in '".concat(config.configPath, "'"));
}

function getGraphQLSchemaPaths(config) {
  return getGraphQLProjects(config).reduce(function (schemas, project) {
    return schemas.concat(resolveSchemaPath(project));
  }, []);
}

function getGraphQLProjectIncludedFilePaths(_x) {
  return _getGraphQLProjectIncludedFilePaths.apply(this, arguments);
}

function _getGraphQLProjectIncludedFilePaths() {
  _getGraphQLProjectIncludedFilePaths = _asyncToGenerator( /*#__PURE__*/_regenerator["default"].mark(function _callee(projectConfig) {
    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return Promise.all(projectConfig.includes.map(function (include) {
              return (0, _util.promisify)(_glob["default"])(resolvePathRelativeToConfig(projectConfig, include), {
                ignore: projectConfig.excludes.map(function (exclude) {
                  return resolvePathRelativeToConfig(projectConfig, exclude);
                })
              });
            }));

          case 2:
            return _context.abrupt("return", _context.sent.reduce(function (allFilePaths, filePaths) {
              return allFilePaths.concat(filePaths);
            }, []));

          case 3:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _getGraphQLProjectIncludedFilePaths.apply(this, arguments);
}

function getGraphQLProjectForSchemaPath(config, schemaPath) {
  var project = getGraphQLProjects(config).filter(function (project) {
    return project.schemaPath === schemaPath;
  }).shift() || config.getProjectConfig();

  if (!project || project.schemaPath !== schemaPath) {
    throw new Error("No project defined in graphql config for schema '".concat(schemaPath, "'"));
  }

  return project;
}