"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getGraphQLProjectForSchemaPath = exports.getGraphQLProjectIncludedFilePaths = exports.getGraphQLSchemaPaths = exports.getGraphQLProjects = exports.resolveSchemaPath = exports.resolveProjectName = exports.resolvePathRelativeToConfig = exports.defaultGraphQLProjectName = void 0;
var tslib_1 = require("tslib");
var fs_1 = require("fs");
var util_1 = require("util");
// we need to use an import/require here because it does not force consumers to
// enable esModuleInterop in tsconfig.json
var glob_1 = tslib_1.__importDefault(require("glob"));
exports.defaultGraphQLProjectName = 'GraphQL';
// temporary utility until `graphql-config` supports this function natively
// see: https://github.com/prisma/graphql-config/pull/113
function resolvePathRelativeToConfig(project, relativePath) {
    return project.resolveConfigPath(relativePath);
}
exports.resolvePathRelativeToConfig = resolvePathRelativeToConfig;
function resolveProjectName(project, defaultName) {
    if (defaultName === void 0) { defaultName = exports.defaultGraphQLProjectName; }
    return project.projectName || defaultName;
}
exports.resolveProjectName = resolveProjectName;
function resolveSchemaPath(project, ignoreMissing) {
    if (ignoreMissing === void 0) { ignoreMissing = false; }
    // schemaPath is nullable in graphq-config even though it cannot actually be
    // omitted. This function simplifies access to the schemaPath without
    // requiring a type guard.
    if (!project.schemaPath) {
        // this case should never happen with a properly formatted config file.
        // graphql-config currently does not perform any validation so it's possible
        // for a mal-formed schema to be loaded at runtime.
        throw new Error("Missing GraphQL schemaPath for project '" + resolveProjectName(project) + "'");
    }
    // resolve fully qualified schemaPath
    var schemaPath = project.resolveConfigPath(project.schemaPath);
    if (ignoreMissing) {
        return schemaPath;
    }
    if (!fs_1.existsSync(schemaPath)) {
        var forProject = project.projectName
            ? " for project '" + project.projectName + "'"
            : '';
        throw new Error([
            "Schema not found" + forProject + ".",
            "Expected to find the schema at '" + schemaPath + "' but the path does not exist.",
            "Check '" + project.configPath + "' and verify that schemaPath is configured correctly" + forProject + ".",
        ].join(' '));
    }
    return schemaPath;
}
exports.resolveSchemaPath = resolveSchemaPath;
function getGraphQLProjects(config) {
    var projects = config.getProjects();
    if (projects) {
        // multi-project configuration, return an array of projects
        return Object.values(projects);
    }
    var project = config.getProjectConfig();
    if (project && project.schemaPath) {
        // single project configuration, return an array of the single project
        return [project];
    }
    // invalid project configuration
    throw new Error("No projects defined in '" + config.configPath + "'");
}
exports.getGraphQLProjects = getGraphQLProjects;
function getGraphQLSchemaPaths(config) {
    return getGraphQLProjects(config).reduce(function (schemas, project) {
        return schemas.concat(resolveSchemaPath(project));
    }, []);
}
exports.getGraphQLSchemaPaths = getGraphQLSchemaPaths;
function getGraphQLProjectIncludedFilePaths(projectConfig) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, Promise.all(projectConfig.includes.map(function (include) {
                        return util_1.promisify(glob_1.default)(resolvePathRelativeToConfig(projectConfig, include), {
                            ignore: projectConfig.excludes.map(function (exclude) {
                                return resolvePathRelativeToConfig(projectConfig, exclude);
                            }),
                        });
                    }))];
                case 1: return [2 /*return*/, (_a.sent()).reduce(function (allFilePaths, filePaths) { return allFilePaths.concat(filePaths); }, [])];
            }
        });
    });
}
exports.getGraphQLProjectIncludedFilePaths = getGraphQLProjectIncludedFilePaths;
function getGraphQLProjectForSchemaPath(config, schemaPath) {
    var project = getGraphQLProjects(config)
        .filter(function (project) { return project.schemaPath === schemaPath; })
        .shift() || config.getProjectConfig();
    if (!project || project.schemaPath !== schemaPath) {
        throw new Error("No project defined in graphql config for schema '" + schemaPath + "'");
    }
    return project;
}
exports.getGraphQLProjectForSchemaPath = getGraphQLProjectForSchemaPath;
